/*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using MKL DFTI in C/C++.
!
!       Double precision split complex in-place 1D transform.
!
!****************************************************************************/

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "mkl_dfti.h"
#include "mkl_service.h"

/* Helper: a relative difference between two complex numbers. */
static double areldiff(double ar,double ai,double br,double bi);

int main(void)
{
    MKL_LONG N = 4096; /* Size of 1D transform */

    DFTI_DESCRIPTOR_HANDLE desc;
    double *x_re, *x_im;
    MKL_LONG n, stride[2];

    stride[0] = 0; /* Offset */
    stride[1] = 1; /* Stride */

    printf("Double precision split complex in-place 1D transform\n"
           "Configuration parameters:\n"
           " DFTI_PRECISION      = DFTI_DOUBLE\n"
           " DFTI_FORWARD_DOMAIN = DFTI_COMPLEX\n"
           " DFTI_PLACEMENT      = DFTI_INPLACE\n"
           " DFTI_DIMENSION      = 1\n"
           " DFTI_LENGTHS        = {%d}\n"
           " DFTI_INPUT_STRIDES  = {%d,%d}\n"
           " DFTI_OUTPUT_STRIDES = {%d,%d}\n",
           (int)N, 
           (int)stride[0], (int)stride[1], 
           (int)stride[0], (int)stride[1]);

    /* Allocate i/o arrays. */
    {
        size_t sz = sizeof(double) * N;
        x_re = (double*) MKL_malloc( sz, 32 );
        x_im = (double*) MKL_malloc( sz, 32 );
        assert(x_re);
        assert(x_im);
    }

    /* Create and commit descriptor. */
    {
        assert(0==DftiCreateDescriptor(&desc,DFTI_DOUBLE,DFTI_COMPLEX,1,N));

        /* Set the split complex layout. */
        assert(0==DftiSetValue(desc,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL));

        assert(0==DftiCommitDescriptor(desc));
    }

    /* Enter input data. */
    for (n = 0; n < N; ++n)
    {
        x_re[n] = sin(n+0.1);
        x_im[n] = sin(n-0.1);
    }

    /* Compute forward transform. */
    assert(0==DftiComputeForward(desc,x_re,x_im));

    /* Compute backward transform. */
    assert(0==DftiComputeBackward(desc,x_re,x_im));

    /* Discard the descriptor. */
    assert(0==DftiFreeDescriptor(&desc));

    /* Check the result: should be simply scaled input. */
    for (n = 0; n < N; ++n)
    {
        double got_re = x_re[n];
        double got_im = x_im[n];
        double exd_re = sin(n+0.1) * N;
        double exd_im = sin(n-0.1) * N;
        if (!(areldiff(got_re,got_im,exd_re,exd_im) < 1e-8))
        {
            printf("Result(%d): "
                   "expected (%lg,%lg), got (%lg,%lg)\n",
                   (int)n, exd_re, exd_im, got_re, got_im);
            printf("TEST FAILED\n");
            exit(1);
        }
    }

    /* Delete the data arrays. */
    MKL_free( x_re );
    MKL_free( x_im );
    printf("TEST PASSED\n");
    return 0;
}

static double areldiff(double ar,double ai,double br,double bi)
{
    /* Use L1 norm for simplicity. */
    double adiff = fabs(ar-br) + fabs(ai-bi);
    double anorm = fabs(ar) + fabs(ai) + fabs(br) + fabs(bi);
    return adiff ? 2*adiff/anorm : adiff;
}
