/*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using MKL DFTI in C/C++.
!
!       Single precision multiple split complex out-of-place 1D transform
!       with non-unit strides.
!
!****************************************************************************/

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "mkl_dfti.h"
#include "mkl_service.h"

#define MAX2(a,b)     ((a)>(b)?(a):(b))

/* Helper: a relative difference between two complex numbers. */
static float areldiff(float ar,float ai,float br,float bi);

int main(void)
{
    MKL_LONG N = 512;           /* Size of transform */
    MKL_LONG M = 4;             /* Number of transforms */

    /* Stride and distance for x: x_re(n,m) = x_re[n*XS + m*XD] */
    MKL_LONG XS = 2;
    MKL_LONG XD = 2*N;

    /* Stride and distance for y: y_re(n,m) = y_re[n*YS + m*YD] */
    MKL_LONG YS = 2*M;
    MKL_LONG YD = 2;

    DFTI_DESCRIPTOR_HANDLE descf,descb;
    float *x_re, *x_im, *y_re, *y_im;
    MKL_LONG n,m,istride[2],ostride[2],idist,odist;

    istride[0] = 0;
    istride[1] = XS;
    idist      = XD;
    ostride[0] = 0;
    ostride[1] = YS;
    odist      = YD;
    printf("Single precision multiple split complex out-of-place 1D transform\n"
           "Configuration parameters:\n"
           " DFTI_PRECISION            = DFTI_SINGLE\n"
           " DFTI_FORWARD_DOMAIN       = DFTI_COMPLEX\n"
           " DFTI_PLACEMENT            = DFTI_NOT_INPLACE\n"
           " DFTI_DIMENSION            = 1\n"
           " DFTI_LENGTHS              = {%d}\n"
           " DFTI_INPUT_STRIDES        = {%d,%d}\n"
           " DFTI_OUTPUT_STRIDES       = {%d,%d}\n"
           " DFTI_NUMBER_OF_TRANSFORMS = %d\n"
           " DFTI_INPUT_DISTANCE       = %d\n"
           " DFTI_OUTPUT_DISTANCE      = %d\n",
           (int)N,
           (int)istride[0],(int)istride[1],
           (int)ostride[0],(int)ostride[1],
           (int)M,
           (int)idist,(int)odist);

    /* Allocate i/o arrays. */
    {
        size_t xsz = sizeof(float) * MAX2(N*XS, M*XD);
        size_t ysz = sizeof(float) * MAX2(N*YS, M*YD);
        x_re = (float*) MKL_malloc( xsz, 32 );
        x_im = (float*) MKL_malloc( xsz, 32 );
        y_re = (float*) MKL_malloc( ysz, 32 );
        y_im = (float*) MKL_malloc( ysz, 32 );
        assert(x_re);
        assert(x_im);
        assert(y_re);
        assert(y_im);
    }

    /* Create descriptor for forward transform: X-->Y. */
    {
        assert(0==DftiCreateDescriptor(&descf,DFTI_SINGLE,DFTI_COMPLEX,1,N));
        assert(0==DftiSetValue(descf,DFTI_PLACEMENT,DFTI_NOT_INPLACE));

        /* Set the split complex layout. */
        assert(0==DftiSetValue(descf,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL));

        assert(0==DftiSetValue(descf,DFTI_INPUT_STRIDES,istride));
        assert(0==DftiSetValue(descf,DFTI_OUTPUT_STRIDES,ostride));

        assert(0==DftiSetValue(descf,DFTI_NUMBER_OF_TRANSFORMS,M));
        assert(0==DftiSetValue(descf,DFTI_INPUT_DISTANCE,idist));
        assert(0==DftiSetValue(descf,DFTI_OUTPUT_DISTANCE,odist));

        assert(0==DftiCommitDescriptor(descf));
    }

    /* Create plan for backward transform: Y-->X. */
    /* Note: input/output strides and distances are exchanged. */
    {
        assert(0==DftiCreateDescriptor(&descb,DFTI_SINGLE,DFTI_COMPLEX,1,N));
        assert(0==DftiSetValue(descb,DFTI_PLACEMENT,DFTI_NOT_INPLACE));

        /* Set the split complex layout. */
        assert(0==DftiSetValue(descb,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL));

        assert(0==DftiSetValue(descb,DFTI_INPUT_STRIDES,ostride));
        assert(0==DftiSetValue(descb,DFTI_OUTPUT_STRIDES,istride));

        assert(0==DftiSetValue(descb,DFTI_NUMBER_OF_TRANSFORMS,M));
        assert(0==DftiSetValue(descb,DFTI_INPUT_DISTANCE,odist));
        assert(0==DftiSetValue(descb,DFTI_OUTPUT_DISTANCE,idist));

        assert(0==DftiCommitDescriptor(descb));
    }

    /* Enter input data. */
    for (m = 0; m < M; ++m)
    {
        for (n = 0; n < N; ++n)
        {
            x_re[n*XS + m*XD] = sinf(n+0.1f);
            x_im[n*XS + m*XD] = sinf(n-0.1f);
        }
    }

    /* Compute forward transform. */
    assert(0==DftiComputeForward(descf,x_re,x_im,y_re,y_im));

    /* Compute backward transform. */
    assert(0==DftiComputeBackward(descb,y_re,y_im,x_re,x_im));

    /* Discard the descriptors. */
    assert(0==DftiFreeDescriptor(&descf));
    assert(0==DftiFreeDescriptor(&descb));

    /* Check the result: should be simply scaled input. */
    for (m = 0; m < M; ++m)
    {
        for (n = 0; n < N; ++n)
        {
            float got_re = x_re[n*XS + m*XD];
            float got_im = x_im[n*XS + m*XD];
            float exd_re = sinf(n+0.1f) * N;
            float exd_im = sinf(n-0.1f) * N;
            if (!(areldiff(got_re,got_im,exd_re,exd_im) < 1e-4f))
            {
                printf("Result(%d;%d): "
                       "expected (%lg,%lg), got (%lg,%lg)\n",
                       (int)n, (int)m, exd_re, exd_im, got_re, got_im);
                printf("TEST FAILED\n");
                exit(1);
            }
        }
    }


    /* Delete the data arrays. */
    MKL_free( x_re );
    MKL_free( x_im );
    MKL_free( y_re );
    MKL_free( y_im );
    printf("TEST PASSED\n");
    return 0;
}

static float areldiff(float ar,float ai,float br,float bi)
{
    /* Use L1 norm for simplicity. */
    float adiff = fabsf(ar-br) + fabsf(ai-bi);
    float anorm = fabsf(ar) + fabsf(ai) + fabsf(br) + fabsf(bi);
    return adiff ? 2*adiff/anorm : adiff;
}
