/*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using MKL DFTI in C/C++.
!
!       Single precision multiple split complex out-of-place 2D transform
!       with non-unit strides.
!
!****************************************************************************/

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "mkl_dfti.h"
#include "mkl_service.h"

#define MAX2(a,b)     ((a)>(b)?(a):(b))
#define MAX3(a,b,c)   MAX2(MAX2(a,b),c)

/* Helper: a relative difference between two complex numbers. */
static float areldiff(float ar,float ai,float br,float bi);

int main(void)
{
    MKL_LONG N1 = 3, N2 = 4;   /* Sizes of 2D transform. */
    MKL_LONG M = 2;            /* Number of transforms. */

    /* Stride and distance for x:
     * x_re(n1,n2,m) = x_re[n1*XS1 + n2*XS2 + m*XD]
     */
    MKL_LONG XS1 = 2, XS2 = 2*N1, XD = 2*N1*N2;

    /* Stride and distance for y:
     * y_re(n1,n2,m) = y_re[n1*YS1 + n2*YS2 + m*YD]
     */
    MKL_LONG YS1 = M*N2*2, YS2 = M*2, YD = 2;

    DFTI_DESCRIPTOR_HANDLE descf,descb;
    float *x_re, *x_im, *y_re, *y_im;
    MKL_LONG n1,n2,m,istride[3],ostride[3],idist,odist;

    istride[0] = 0;
    istride[1] = XS1;
    istride[2] = XS2;
    idist      = XD;
    ostride[0] = 0;
    ostride[1] = YS1;
    ostride[2] = YS2;
    odist      = YD;
    printf("Single precision multiple split complex out-of-place 2D transform\n"
           "Configuration parameters:\n"
           " DFTI_PRECISION            = DFTI_SINGLE\n"
           " DFTI_FORWARD_DOMAIN       = DFTI_COMPLEX\n"
           " DFTI_PLACEMENT            = DFTI_NOT_INPLACE\n"
           " DFTI_DIMENSION            = 2\n"
           " DFTI_LENGTHS              = {%d,%d}\n"
           " DFTI_INPUT_STRIDES        = {%d,%d,%d}\n"
           " DFTI_OUTPUT_STRIDES       = {%d,%d,%d}\n"
           " DFTI_NUMBER_OF_TRANSFORMS = %d\n"
           " DFTI_INPUT_DISTANCE       = %d\n"
           " DFTI_OUTPUT_DISTANCE      = %d\n",
           (int)N1,(int)N2,
           (int)istride[0],(int)istride[1],(int)istride[2],
           (int)ostride[0],(int)ostride[1],(int)ostride[2],
           (int)M,
           (int)idist,(int)odist);

    /* Allocate i/o arrays. */
    {
        size_t xsz = sizeof(float) * MAX3(N1*XS1, N2*XS2, M*XD);
        size_t ysz = sizeof(float) * MAX3(N1*YS1, N2*YS2, M*YD);
        x_re = (float*) MKL_malloc( xsz, 32 );
        x_im = (float*) MKL_malloc( xsz, 32 );
        y_re = (float*) MKL_malloc( ysz, 32 );
        y_im = (float*) MKL_malloc( ysz, 32 );
        assert(x_re);
        assert(x_im);
        assert(y_re);
        assert(y_im);
    }

    /* Create descriptor for forward transform: X-->Y */
    {
        MKL_LONG N[2];
        N[0] = N1;
        N[1] = N2;

        assert(0==DftiCreateDescriptor(&descf,DFTI_SINGLE,DFTI_COMPLEX,2,N));
        assert(0==DftiSetValue(descf,DFTI_PLACEMENT,DFTI_NOT_INPLACE));

        /* Set the split complex layout. */
        assert(0==DftiSetValue(descf,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL));

        assert(0==DftiSetValue(descf,DFTI_INPUT_STRIDES,istride));
        assert(0==DftiSetValue(descf,DFTI_OUTPUT_STRIDES,ostride));

        assert(0==DftiSetValue(descf,DFTI_NUMBER_OF_TRANSFORMS,M));
        assert(0==DftiSetValue(descf,DFTI_INPUT_DISTANCE,idist));
        assert(0==DftiSetValue(descf,DFTI_OUTPUT_DISTANCE,odist));

        assert(0==DftiCommitDescriptor(descf));
    }

    /* Create descriptor for backward transform: Y-->X */
    /* Note: input/output strides and distances are exchanged. */
    {
        MKL_LONG N[2];
        N[0] = N1;
        N[1] = N2;

        assert(0==DftiCreateDescriptor(&descb,DFTI_SINGLE,DFTI_COMPLEX,2,N));
        assert(0==DftiSetValue(descb,DFTI_PLACEMENT,DFTI_NOT_INPLACE));

        /* Set the split complex layout. */
        assert(0==DftiSetValue(descb,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL));

        assert(0==DftiSetValue(descb,DFTI_INPUT_STRIDES,ostride));
        assert(0==DftiSetValue(descb,DFTI_OUTPUT_STRIDES,istride));

        assert(0==DftiSetValue(descb,DFTI_NUMBER_OF_TRANSFORMS,M));
        assert(0==DftiSetValue(descb,DFTI_INPUT_DISTANCE,odist));
        assert(0==DftiSetValue(descb,DFTI_OUTPUT_DISTANCE,idist));

        assert(0==DftiCommitDescriptor(descb));
    }

    /* Enter input data. */
    for (m = 0; m < M; ++m)
    {
        for (n1 = 0; n1 < N1; ++n1)
        {
            for (n2 = 0; n2 < N2; ++n2)
            {
                x_re[n1*XS1 + n2*XS2 + m*XD] = sinf(n1+n2+0.1f);
                x_im[n1*XS1 + n2*XS2 + m*XD] = sinf(n1+n2-0.1f);
            }
        }
    }

    /* Compute forward transform. */
    assert(0==DftiComputeForward(descf,x_re,x_im,y_re,y_im));

    /* Compute backward transform. */
    assert(0==DftiComputeBackward(descb,y_re,y_im,x_re,x_im));

    /* Discard the descriptors. */
    assert(0==DftiFreeDescriptor(&descf));
    assert(0==DftiFreeDescriptor(&descb));

    /* Check the result: should be simply scaled input. */
    for (m = 0; m < M; ++m)
    {
        for (n1 = 0; n1 < N1; ++n1)
        {
            for (n2 = 0; n2 < N2; ++n2)
            {
                float got_re = x_re[n1*XS1 + n2*XS2 + m*XD];
                float got_im = x_im[n1*XS1 + n2*XS2 + m*XD];
                float exd_re = sinf(n1+n2+0.1f) * N1 * N2;
                float exd_im = sinf(n1+n2-0.1f) * N1 * N2;
                if (!(areldiff(got_re,got_im,exd_re,exd_im) < 1e-4f))
                {
                    printf("Result(%d,%d;%d): "
                           "expected (%lg,%lg), got (%lg,%lg)\n",
                           (int)n1, (int)n2, (int)m,
                           exd_re, exd_im, got_re, got_im);
                    printf("TEST FAILED\n");
                    exit(1);
                }
            }
        }
    }

    /* Delete the data arrays. */
    MKL_free( x_re );
    MKL_free( x_im );
    MKL_free( y_re );
    MKL_free( y_im );
    printf("TEST PASSED\n");
    return 0;
}

static float areldiff(float ar,float ai,float br,float bi)
{
    /* Use L1 norm for simplicity. */
    float adiff = fabsf(ar-br) + fabsf(ai-bi);
    float anorm = fabsf(ar) + fabsf(ai) + fabsf(br) + fabsf(bi);
    return adiff ? 2*adiff/anorm : adiff;
}
