/*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using MKL DFTI in C/C++.
!
!       Single precision split complex in-place 3D transform.
!
!****************************************************************************/

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "mkl_dfti.h"
#include "mkl_service.h"

/* Helper: a relative difference between two complex numbers. */
static float areldiff(float ar,float ai,float br,float bi);

int main(void)
{
    /* Sizes of 3D transform. */
    MKL_LONG N1 = 3, N2 = 4, N3 = 5;

    DFTI_DESCRIPTOR_HANDLE desc;
    float *x_re, *x_im;
    MKL_LONG n1, n2, n3, stride[4];

    stride[0] = 0;              /* Offset */
    stride[1] = N2*N1;          /* Stride along N3 */
    stride[2] = N1;             /* Stride along N2 */
    stride[3] = 1;              /* Stride along N1 */

    printf("Single precision split complex in-place 3D transform\n"
           "Configuration parameters:\n"
           " DFTI_PRECISION      = DFTI_SINGLE\n"
           " DFTI_FORWARD_DOMAIN = DFTI_COMPLEX\n"
           " DFTI_PLACEMENT      = DFTI_INPLACE\n"
           " DFTI_DIMENSION      = 3\n"
           " DFTI_LENGTHS        = {%d,%d,%d}\n"
           " DFTI_INPUT_STRIDES  = {%d,%d,%d,%d}\n"
           " DFTI_OUTPUT_STRIDES = {%d,%d,%d,%d}\n",
           (int)N3,(int)N2,(int)N1,
           (int)stride[0], (int)stride[1], (int)stride[2], (int)stride[3],
           (int)stride[0], (int)stride[1], (int)stride[2], (int)stride[3]);

    /* Allocate i/o arrays. */
    {
        size_t sz = sizeof(float) * N1 * N2 * N3;
        x_re = (float*) MKL_malloc( sz, 32 );
        x_im = (float*) MKL_malloc( sz, 32 );
        assert(x_re);
        assert(x_im);
    }

    /* Create and commit descriptor. */
    {
        MKL_LONG N[3];
        N[0] = N3;
        N[1] = N2;
        N[2] = N1;

        assert(0==DftiCreateDescriptor(&desc,DFTI_SINGLE,DFTI_COMPLEX,3,N));

        /* Set the split complex layout. */
        assert(0==DftiSetValue(desc,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL));

        assert(0==DftiCommitDescriptor(desc));
    }

    /* Enter input data. */
    for (n1 = 0; n1 < N1; ++n1)
    {
        for (n2 = 0; n2 < N2; ++n2)
        {
            for (n3 = 0; n3 < N3; ++n3)
            {
                x_re[n1*N2*N3 + n2*N3 + n3] = sinf(n1+n2+n3+0.1f);
                x_im[n1*N2*N3 + n2*N3 + n3] = sinf(n1+n2+n3-0.1f);
            }
        }
    }

    /* Compute forward transform. */
    assert(0==DftiComputeForward(desc,x_re,x_im));

    /* Compute backward transform with new-array function. */
    assert(0==DftiComputeBackward(desc,x_re,x_im));

    /* Discard the descriptor. */
    assert(0==DftiFreeDescriptor(&desc));

    /* Check the result: should be simply scaled input. */
    for (n1 = 0; n1 < N1; ++n1)
    {
        for (n2 = 0; n2 < N2; ++n2)
        {
            for (n3 = 0; n3 < N3; ++n3)
            {
                float got_re = x_re[n1*N2*N3 + n2*N3 + n3];
                float got_im = x_im[n1*N2*N3 + n2*N3 + n3];
                float exd_re = sinf(n1+n2+n3+0.1f) * N1 * N2 * N3;
                float exd_im = sinf(n1+n2+n3-0.1f) * N1 * N2 * N3;
                if (!(areldiff(got_re,got_im,exd_re,exd_im) < 1e-4f))
                {
                    printf("Result(%d,%d,%d): "
                           "expected (%lg,%lg), got (%lg,%lg)\n",
                           (int)n1, (int)n2, (int)n3,
                           exd_re, exd_im, got_re, got_im);
                    printf("TEST FAILED\n");
                    exit(1);
                }
            }
        }
    }

    /* Delete the data arrays. */
    MKL_free( x_re );
    MKL_free( x_im );
    printf("TEST PASSED\n");
    return 0;
}

static float areldiff(float ar,float ai,float br,float bi)
{
    /* Use L1 norm for simplicity. */
    float adiff = fabsf(ar-br) + fabsf(ai-bi);
    float anorm = fabsf(ar) + fabsf(ai) + fabsf(br) + fabsf(bi);
    return adiff ? 2*adiff/anorm : adiff;
}
