!*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using MKL DFTI in Fortran.
!
!       Double precision split complex in-place 1D transform.
!
!*****************************************************************************

program double_split_complex_1d_ex1

  use mkl_dfti, forget => DFTI_DOUBLE, DFTI_DOUBLE => DFTI_DOUBLE_R

  integer, parameter :: N = 128 ! Size of 1D transform

  type(dfti_descriptor), pointer :: desc
  real*8, allocatable :: xr(:), xi(:)
  complex*16 :: got, exd
  integer i, stride(2), status

  stride(1) = 0                 ! Offset
  stride(2) = 1                 ! Stride

  print *,"Double precision split complex in-place 1D transform"
  print *,"Configuration parameters:"
  print *," DFTI_PRECISION      = DFTI_DOUBLE"
  print *," DFTI_FORWARD_DOMAIN = DFTI_COMPLEX"
  print *," DFTI_PLACEMENT      = DFTI_INPLACE"
  print *," DFTI_DIMENSION      = 1"
  print '("  DFTI_LENGTHS        = "I0)', N
  print '("  DFTI_INPUT_STRIDES  = (/"I0","I0"/)")',stride
  print '("  DFTI_OUTPUT_STRIDES = (/"I0","I0"/)")',stride

  ! Allocate i/o arrays.
  allocate(xr(N),xi(N))

  ! Create descriptor.
  status = DftiCreateDescriptor(desc,DFTI_DOUBLE,DFTI_COMPLEX,1,N)
  if (status /= 0) call nogo("DftiCreateDescriptor",status)

  status = DftiSetValue(desc,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL)
  if (status /= 0) call nogo("DftiSetValue(COMPLEX_STORAGE)",status)

  status = DftiCommitDescriptor(desc)
  if (status /= 0) call nogo("DftiCommitDescriptor",status)

  ! Enter input data.
  do i = 1, N
     xr(i) = sin(n+0.1d0)
     xi(i) = sin(n-0.1d0)
  enddo

  ! Compute forward transform.
  status = DftiComputeForward(desc,xr,xi)
  if (status /= 0) call nogo("DftiComputeForward",status)

  ! Compute backward transform.
  status = DftiComputeBackward(desc,xr,xi)
  if (status /= 0) call nogo("DftiComputeBackward",status)

  ! Discard the descriptor.
  status = DftiFreeDescriptor(desc)
  if (status /= 0) call nogo("DftiFreeDescriptor",status)

  ! Check the result: should be simply scaled input.
  do i = 1, N
     got = dcmplx(xr(i),xi(i))
     exd = dcmplx(sin(n+0.1d0),sin(n-0.1d0))*N
     if (.not.(areldiff(got,exd) < 1d-8)) then
        call error(i,exd,got)
     endif
  enddo

  ! Deallocate data arrays.
  deallocate( xr, xi )
  print *, 'TEST PASSED'
  stop 0

contains

  ! A relative difference between two complex numbers
  real*8 function areldiff(a,b)
    complex*16 a,b
    real*8 norm
    norm = max(abs(a),abs(b))
    if (norm > 0) then
       areldiff = abs(a-b) / norm
    else
       areldiff = abs(a-b)
    endif
  end function areldiff

  ! Report correctness error and terminate
  subroutine error(i,exd,got)
    integer :: i
    complex*16 :: exd, got

    print '("Result("I0"): "$)', i
    print '("expected ("G15.8","G15.8") "$)', exd
    print '("got ("G15.8","G15.8")")', got
    print *, 'TEST FAILED'
    stop 1
  end subroutine error

  ! Report execution error and terminate
  subroutine nogo(string,status)
    character*(*) string
    integer status
    print *,"Cannot continue: ",string," failed with status ", status
    stop 1
  end subroutine nogo

end program double_split_complex_1d_ex1
