!*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using MKL DFTI in Fortran.
!
!       Double precision split complex in-place 3D transform.
!
!*****************************************************************************

program double_split_complex_3d_ex1

  use mkl_dfti, forget => DFTI_DOUBLE, DFTI_DOUBLE => DFTI_DOUBLE_R

  integer, parameter :: N(3) = (/16,32,8/) ! Sizes of 3D transform

  type(dfti_descriptor), pointer :: desc
  real*8, allocatable :: xr(:,:,:), xi(:,:,:)
  complex*16 :: got, exd
  integer n1, n2, n3, stride(4), status

  stride(1) = 0                 ! Offset
  stride(2) = 1                 ! Stride along N(1)
  stride(3) = N(1)              ! Stride along N(2)
  stride(4) = N(1)*N(2)         ! Stride along N(3)

  print *,"Double precision split complex in-place 3D transform"
  print *,"Configuration parameters:"
  print *," DFTI_PRECISION      = DFTI_DOUBLE"
  print *," DFTI_FORWARD_DOMAIN = DFTI_COMPLEX"
  print *," DFTI_PLACEMENT      = DFTI_INPLACE"
  print *," DFTI_DIMENSION      = 3"
  print '("  DFTI_LENGTHS        = (/"I0","I0","I0"/)")', N
  print '("  DFTI_INPUT_STRIDES  = (/"I0,3(","I0)"/)")',stride
  print '("  DFTI_OUTPUT_STRIDES = (/"I0,3(","I0)"/)")',stride

  ! Allocate i/o arrays.
  allocate(xr(N(1),N(2),N(3)),xi(N(1),N(2),N(3)))

  ! Create descriptor.
  status = DftiCreateDescriptor(desc,DFTI_DOUBLE,DFTI_COMPLEX,3,N)
  if (status /= 0) call nogo("DftiCreateDescriptor",status)

  status = DftiSetValue(desc,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL)
  if (status /= 0) call nogo("DftiSetValue(COMPLEX_STORAGE)",status)

  status = DftiCommitDescriptor(desc)
  if (status /= 0) call nogo("DftiCommitDescriptor",status)

  ! Enter input data.
  do n1 = 1, N(1)
     do n2 = 1, N(2)
        do n3 = 1, N(3)
           xr(n1,n2,n3) = sin(n1+n2+n3+0.1d0)
           xi(n1,n2,n3) = sin(n1+n2+n3-0.1d0)
        enddo
     enddo
  enddo

  ! Compute forward transform.
  status = DftiComputeForward(desc,xr(:,1,1),xi(:,1,1))
  if (status /= 0) call nogo("DftiComputeForward",status)

  ! Compute backward transform.
  status = DftiComputeBackward(desc,xr(:,1,1),xi(:,1,1))
  if (status /= 0) call nogo("DftiComputeBackward",status)

  ! Discard the descriptor.
  status = DftiFreeDescriptor(desc)
  if (status /= 0) call nogo("DftiFreeDescriptor",status)

  ! Check the result: should be simply scaled input.
  do n1 = 1, N(1)
     do n2 = 1, N(2)
        do n3 = 1, N(3)
           got = dcmplx(xr(n1,n2,n3),xi(n1,n2,n3))
           exd = dcmplx(sin(n1+n2+n3+0.1d0),sin(n1+n2+n3-0.1d0)) &
                & *N(1)*N(2)*N(3)
           if (.not.(areldiff(got,exd) < 1d-8)) then
              call error(n1,n2,n3,exd,got)
           endif
        enddo
     enddo
  enddo

  ! Deallocate data arrays.
  deallocate( xr, xi )
  print *, 'TEST PASSED'
  stop 0

contains

  ! A relative difference between two complex numbers
  real*8 function areldiff(a,b)
    complex*16 a,b
    real*8 norm
    norm = max(abs(a),abs(b))
    if (norm > 0) then
       areldiff = abs(a-b) / norm
    else
       areldiff = abs(a-b)
    endif
  end function areldiff

  ! Report error and terminate
  subroutine error(n1,n2,n3,exd,got)
    integer :: n1,n2,n3
    complex*16 :: exd, got

    print '("Result("I0","I0","I0"): "$)', n1,n2,n3
    print '("expected ("G15.8","G15.8") "$)', exd
    print '("got ("G15.8","G15.8")")', got
    print *, 'TEST FAILED'
    stop 1
  end subroutine error

  ! Report execution error and terminate
  subroutine nogo(string,status)
    character*(*) string
    integer status
    print *,"Cannot continue: ",string," failed with status ", status
    stop 1
  end subroutine nogo


end program double_split_complex_3d_ex1
