!*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using MKL DFTI in Fortran.
!
!       Single precision multiple split complex out-of-place 1D transform.
!
!*****************************************************************************

program single_split_complex_1d_ex2

  use mkl_dfti, forget => DFTI_SINGLE, DFTI_SINGLE => DFTI_SINGLE_R

  integer, parameter :: N = 25  ! Size of transform
  integer, parameter :: M = 10  ! Number of transforms

  type(dfti_descriptor), pointer :: descf,descb
  real*4, allocatable :: xr(:,:), xi(:,:), yr(:,:), yi(:,:)
  complex*8 :: got, exd
  integer i, j, xstride(2), ystride(2), xdist, ydist, status

  ! Allocate i/o arrays.
  allocate(xr(N,M),xi(N,M))
  allocate(yr(M,N),yi(M,N))

  ! Define strides and distances.
  xstride(1) = 0
  xstride(2) = 1
  xdist      = size(xr,dim=1)
  ystride(1) = 0
  ystride(2) = size(yr,dim=1)
  ydist      = 1

  print *,"Single precision multiple split complex out-of-place 1D transform"
  print *,"Configuration parameters:"
  print *," DFTI_PRECISION            = DFTI_SINGLE"
  print *," DFTI_FORWARD_DOMAIN       = DFTI_COMPLEX"
  print *," DFTI_PLACEMENT            = DFTI_NOT_INPLACE"
  print *," DFTI_DIMENSION            = 1"
  print '("  DFTI_LENGTHS              = "I0)', N
  print '("  DFTI_INPUT_STRIDES        = (/"I0","I0"/)")',xstride
  print '("  DFTI_OUTPUT_STRIDES       = (/"I0","I0"/)")',ystride
  print '("  DFTI_NUMBER_OF_TRANSFORMS = "I0)', M
  print '("  DFTI_INPUT_DISTANCE       = "I0)', xdist
  print '("  DFTI_OUTPUT_DISTANCE      = "I0)', ydist

  ! Create descriptor for forward transform.
  status = DftiCreateDescriptor(descf,DFTI_SINGLE,DFTI_COMPLEX,1,N)
  if (status /= 0) call nogo("DftiCreateDescriptor(descf)",status)

  status = DftiSetValue(descf,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL)
  if (status /= 0) call nogo("DftiSetValue(descf,COMPLEX_STORAGE)",status)

  status = DftiSetValue(descf,DFTI_PLACEMENT,DFTI_NOT_INPLACE)
  if (status /= 0) call nogo("DftiSetValue(descf,PLACEMENT)",status)

  status = DftiSetValue(descf,DFTI_INPUT_STRIDES,xstride)
  if (status /= 0) call nogo("DftiSetValue(descf,INPUT_STRIDES)",status)

  status = DftiSetValue(descf,DFTI_OUTPUT_STRIDES,ystride)
  if (status /= 0) call nogo("DftiSetValue(descf,OUTPUT_STRIDES)",status)

  status = DftiSetValue(descf,DFTI_NUMBER_OF_TRANSFORMS,M)
  if (status /= 0) call nogo("DftiSetValue(descf,NUMBER_OF_TRANSFORMS)",status)

  status = DftiSetValue(descf,DFTI_INPUT_DISTANCE,xdist)
  if (status /= 0) call nogo("DftiSetValue(descf,INPUT_DISTANCE)",status)

  status = DftiSetValue(descf,DFTI_OUTPUT_DISTANCE,ydist)
  if (status /= 0) call nogo("DftiSetValue(descf,OUTPUT_DISTANCE)",status)

  status = DftiCommitDescriptor(descf)
  if (status /= 0) call nogo("DftiCommitDescriptor(descf)",status)

  ! Create descriptor for backward transform.
  ! Note: input/output strides and distances are exchanged.
  status = DftiCreateDescriptor(descb,DFTI_SINGLE,DFTI_COMPLEX,1,N)
  if (status /= 0) call nogo("DftiCreateDescriptor(descb)",status)

  status = DftiSetValue(descb,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL)
  if (status /= 0) call nogo("DftiSetValue(descb,COMPLEX_STORAGE)",status)

  status = DftiSetValue(descb,DFTI_PLACEMENT,DFTI_NOT_INPLACE)
  if (status /= 0) call nogo("DftiSetValue(descb,PLACEMENT)",status)

  status = DftiSetValue(descb,DFTI_INPUT_STRIDES,ystride)
  if (status /= 0) call nogo("DftiSetValue(descb,INPUT_STRIDES)",status)

  status = DftiSetValue(descb,DFTI_OUTPUT_STRIDES,xstride)
  if (status /= 0) call nogo("DftiSetValue(descb,OUTPUT_STRIDES)",status)

  status = DftiSetValue(descb,DFTI_NUMBER_OF_TRANSFORMS,M)
  if (status /= 0) call nogo("DftiSetValue(descb,NUMBER_OF_TRANSFORMS)",status)

  status = DftiSetValue(descb,DFTI_INPUT_DISTANCE,ydist)
  if (status /= 0) call nogo("DftiSetValue(descb,INPUT_DISTANCE)",status)

  status = DftiSetValue(descb,DFTI_OUTPUT_DISTANCE,xdist)
  if (status /= 0) call nogo("DftiSetValue(descb,OUTPUT_DISTANCE)",status)

  status = DftiCommitDescriptor(descb)
  if (status /= 0) call nogo("DftiCommitDescriptor(descb)",status)

  ! Enter input data.
  do i = 1, N
     xr(i,:) = sin(i+0.1)
     xi(i,:) = sin(i-0.1)
  enddo

  ! Compute forward transform.
  status = DftiComputeForward(descf,xr(:,1),xi(:,1),yr(:,1),yi(:,1))
  if (status /= 0) call nogo("DftiComputeForward(descf)",status)

  ! Compute backward transform.
  status = DftiComputeBackward(descb,yr(:,1),yi(:,1),xr(:,1),xi(:,1))
  if (status /= 0) call nogo("DftiComputeForward",status)

  ! Discard the descriptors.
  status = DftiFreeDescriptor(descf)
  if (status /= 0) call nogo("DftiFreeDescriptor(descf)",status)
  status = DftiFreeDescriptor(descb)
  if (status /= 0) call nogo("DftiFreeDescriptor(descb)",status)

  ! Check the result: should be simply scaled input.
  do i = 1, N
     do j = 1, M
        got = cmplx(xr(i,j),xi(i,j))
        exd = cmplx(sin(i+0.1),sin(i-0.1))*N
        if (.not.(areldiff(got,exd) < 1e-4)) then
           call error(i,j,exd,got)
        endif
     enddo
  enddo

  ! Deallocate data arrays.
  deallocate( xr, xi )
  deallocate( yr, yi )
  print *, 'TEST PASSED'
  stop 0

contains

  ! A relative difference between two complex numbers
  real*4 function areldiff(a,b)
    complex*8 a,b
    real*4 norm
    norm = max(abs(a),abs(b))
    if (norm > 0) then
       areldiff = abs(a-b) / norm
    else
       areldiff = abs(a-b)
    endif
  end function areldiff

  ! Report error and terminate
  subroutine error(i,j,exd,got)
    integer :: i,j
    complex*8 :: exd, got

    print '("Result("I0";"I0"): "$)', i,j
    print '("expected ("G11.4","G11.4") "$)', exd
    print '("got ("G11.4","G11.4")")', got
    print *, 'TEST FAILED'
    stop 1
  end subroutine error

  ! Report execution error and terminate
  subroutine nogo(string,status)
    character*(*) string
    integer status
    print *,"Cannot continue: ",string," failed with status ", status
    stop 1
  end subroutine nogo

end program single_split_complex_1d_ex2
