!*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using MKL DFTI in Fortran.
!
!       Single precision split complex in-place 2D transform.
!
!*****************************************************************************

program single_split_complex_2d_ex1

  use mkl_dfti, forget => DFTI_SINGLE, DFTI_SINGLE => DFTI_SINGLE_R

  integer, parameter :: N(2) = (/10,20/) ! Sizes of 2D transform

  type(dfti_descriptor), pointer :: desc
  real*4, allocatable :: xr(:,:), xi(:,:)
  complex*8 :: got, exd
  integer n1, n2, stride(3), status

  stride(1) = 0                 ! Offset
  stride(2) = 1                 ! Stride along N(1)
  stride(3) = N(1)              ! Stride along N(2)

  print *,"Single precision split complex in-place 2D transform"
  print *,"Configuration parameters:"
  print *," DFTI_PRECISION      = DFTI_SINGLE"
  print *," DFTI_FORWARD_DOMAIN = DFTI_COMPLEX"
  print *," DFTI_PLACEMENT      = DFTI_INPLACE"
  print *," DFTI_DIMENSION      = 2"
  print '("  DFTI_LENGTHS        = (/"I0","I0"/)")', N
  print '("  DFTI_INPUT_STRIDES  = (/"I0","I0","I0"/)")',stride
  print '("  DFTI_OUTPUT_STRIDES = (/"I0","I0","I0"/)")',stride

  ! Allocate i/o arrays.
  allocate(xr(N(1),N(2)),xi(N(1),N(2)))

  ! Create descriptor.
  status = DftiCreateDescriptor(desc,DFTI_SINGLE,DFTI_COMPLEX,2,N)
  if (status /= 0) call nogo("DftiCreateDescriptor",status)

  status = DftiSetValue(desc,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL)
  if (status /= 0) call nogo("DftiSetValue(COMPLEX_STORAGE)",status)

  status = DftiCommitDescriptor(desc)
  if (status /= 0) call nogo("DftiCommitDescriptor",status)

  ! Enter input data.
  do n1 = 1, N(1)
     do n2 = 1, N(2)
        xr(n1,n2) = sin(n1+n2+0.1)
        xi(n1,n2) = sin(n1+n2-0.1)
     enddo
  enddo

  ! Compute forward transform.
  status = DftiComputeForward(desc,xr(:,1),xi(:,1))
  if (status /= 0) call nogo("DftiComputeForward",status)

  ! Compute backward transform.
  status = DftiComputeBackward(desc,xr(:,1),xi(:,1))
  if (status /= 0) call nogo("DftiComputeBackward",status)

  ! Discard the descriptor.
  status = DftiFreeDescriptor(desc)
  if (status /= 0) call nogo("DftiFreeDescriptor",status)

  ! Check the result: should be simply scaled input.
  do n1 = 1, N(1)
     do n2 = 1, N(2)
        got = cmplx(xr(n1,n2),xi(n1,n2))
        exd = cmplx(sin(n1+n2+0.1),sin(n1+n2-0.1))*N(1)*N(2)
        if (.not.(areldiff(got,exd) < 1e-4)) then
           call error(n1,n2,exd,got)
        endif
     enddo
  enddo


  ! Deallocate data arrays.
  deallocate( xr, xi )
  print *, 'TEST PASSED'
  stop 0

contains

  ! A relative difference between two complex numbers
  real*4 function areldiff(a,b)
    complex*8 a,b
    real*4 norm
    norm = max(abs(a),abs(b))
    if (norm > 0) then
       areldiff = abs(a-b) / norm
    else
       areldiff = abs(a-b)
    endif
  end function areldiff

  ! Report error and terminate
  subroutine error(n1,n2,exd,got)
    integer :: n1,n2
    complex*8 :: exd, got
    
    print '("Result("I0","I0"): "$)', n1,n2
    print '("expected ("G11.4","G11.4") "$)', exd
    print '("got ("G11.4","G11.4")")', got
    print *, 'TEST FAILED'
    stop 1
  end subroutine error

  ! Report execution error and terminate
  subroutine nogo(string,status)
    character*(*) string
    integer status
    print *,"Cannot continue: ",string," failed with status ", status
    stop 1
  end subroutine nogo

end program single_split_complex_2d_ex1
