/*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using FFTW3 interface to Intel MKL (C-interface).
!
!       Double precision split complex in-place 1D transform
!       using fftw_plan_guru_split_dft().
!
!****************************************************************************/

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "fftw3.h"

/* Helper: a relative difference between two complex numbers. */
static double areldiff(double ar,double ai,double br,double bi);

int main(void)
{
    int N = 1024; /* Size of 1D transform */

    fftw_plan plan;
    double *x_re, *x_im;
    int n;

    printf("Double precision split complex in-place\n"
           " 1D transform of length %d\n"
           " using fftw_plan_guru_split_dft()\n",
           N);

    /* Allocate i/o arrays. */
    {
        size_t sz = sizeof(double) * N;
        x_re = (double*) fftw_malloc( sz );
        x_im = (double*) fftw_malloc( sz );
        assert(x_re);
        assert(x_im);
    }

    /* Create forward transform plan. */
    {
        fftw_iodim dim;
        dim.n = N;
        dim.is = dim.os = 1;
        plan = fftw_plan_guru_split_dft(1, &dim, 0, NULL,
                                        x_re, x_im, x_re, x_im,
                                        FFTW_ESTIMATE);
        assert(plan);
    }

    /* Enter input data. */
    /* Note: this should be done after the plan is created. */
    for (n = 0; n < N; ++n)
    {
        x_re[n] = sin(n+0.1);
        x_im[n] = sin(n-0.1);
    }

    /* Compute forward transform. */
    fftw_execute(plan);

    /* Compute backward transform with new-array function. */
    /* Note: we use the same plan, but exchange Re<->Im. */
    fftw_execute_split_dft(plan,x_im,x_re,x_im,x_re);

    /* Destroy the plan. */
    fftw_destroy_plan(plan);

    /* Check the result: should be simply scaled input. */
    for (n = 0; n < N; ++n)
    {
        double got_re = x_re[n];
        double got_im = x_im[n];
        double exd_re = sin(n+0.1)*N;
        double exd_im = sin(n-0.1)*N;
        if (!(areldiff(got_re,got_im,exd_re,exd_im) < 1e-8))
        {
            printf("Result(%d): "
                   "expected (%lg,%lg), got (%lg,%lg)\n",
                   n, exd_re, exd_im, got_re, got_im);
            printf("TEST FAILED\n");
            exit(1);
        }
    }

    /* Delete the data arrays. */
    fftw_free( x_re );
    fftw_free( x_im );
    printf("TEST PASSED\n");
    return 0;
}

static double areldiff(double ar,double ai,double br,double bi)
{
    /* Use L1 norm for simplicity. */
    double adiff = fabs(ar-br) + fabs(ai-bi);
    double anorm = fabs(ar) + fabs(ai) + fabs(br) + fabs(bi);
    return adiff ? 2*adiff/anorm : adiff;
}
