/*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using FFTW3 interface to Intel MKL (C-interface).
!
!       Double precision multiple split complex out-of-place 1D transform
!       with non-unit strides using fftw_plan_guru64_split_dft().
!
!****************************************************************************/

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "fftw3.h"

#define MAX2(a,b)     ((a)>(b)?(a):(b))

/* Helper: a relative difference between two complex numbers. */
static double areldiff(double ar,double ai,double br,double bi);

int main(void)
{
    int N = 512;                /* Size of transform */
    int M = 4;                  /* Number of transforms */

    /* Stride and distance for x: x_re(n,m) = x_re[n*XS + m*XD] */
    int XS = 1;
    int XD = N;

    /* Stride and distance for y: y_re(n,m) = y_re[n*YS + m*YD] */
    int YS = 2*M;
    int YD = 2;

    fftw_plan planf,planb;
    double *x_re, *x_im, *y_re, *y_im;
    int n,m;

    printf("Double precision multiple split complex out-of-place\n"
           " 1D transform %d:%d:%d*%d:%d:%d\n"
           " using fftw_plan_guru64_split_dft()\n",
           N,XS,YS,M,XD,YD);

    /* Allocate i/o arrays. */
    {
        size_t xsz = sizeof(double) * MAX2(N*XS, M*XD);
        size_t ysz = sizeof(double) * MAX2(N*YS, M*YD);
        x_re = (double*) fftw_malloc( xsz );
        x_im = (double*) fftw_malloc( xsz );
        y_re = (double*) fftw_malloc( ysz );
        y_im = (double*) fftw_malloc( ysz );
        assert(x_re);
        assert(x_im);
        assert(y_re);
        assert(y_im);
    }

    /* Create plan for forward transform: X-->Y. */
    {
        fftw_iodim64 dim, howmany;
        dim.n = N;
        dim.is = XS;
        dim.os = YS;
        howmany.n = M;
        howmany.is = XD;
        howmany.os = YD;
        planf = fftw_plan_guru64_split_dft(1, &dim, 1, &howmany,
                                           x_re, x_im, y_re, y_im,
                                           FFTW_ESTIMATE);
        assert(planf);
    }

    /* Create plan for backward transform: Y-->X. */
    {
        fftw_iodim64 dim, howmany;

        /* Note: we exchange input and output strides/distances. */
        dim.n = N;
        dim.is = YS;
        dim.os = XS;
        howmany.n = M;
        howmany.is = YD;
        howmany.os = XD;

        planb = fftw_plan_guru64_split_dft(1, &dim, 1, &howmany,
                                           y_im, y_re, x_im, x_re,
                                           FFTW_ESTIMATE);
        assert(planb);
    }

    /* Enter input data. */
    /* Note: input should be initialized after the plan is created. */
    for (m = 0; m < M; ++m)
    {
        for (n = 0; n < N; ++n)
        {
            x_re[n*XS + m*XD] = sin(n+0.1);
            x_im[n*XS + m*XD] = sin(n-0.1);
        }
    }

    /* Compute forward transform. */
    fftw_execute(planf);

    /* Compute backward transform. */
    fftw_execute(planb);

    /* Destroy the plans. */
    fftw_destroy_plan(planf);
    fftw_destroy_plan(planb);

    /* Check the result: should be simply scaled input. */
    for (m = 0; m < M; ++m)
    {
        for (n = 0; n < N; ++n)
        {
            double got_re = x_re[n*XS + m*XD];
            double got_im = x_im[n*XS + m*XD];
            double exd_re = sin(n+0.1) * N;
            double exd_im = sin(n-0.1) * N;
            if (!(areldiff(got_re,got_im,exd_re,exd_im) < 1e-8))
            {
                printf("Result(%d;%d): "
                       "expected (%lg,%lg), got (%lg,%lg)\n",
                       n, m, exd_re, exd_im, got_re, got_im);
                printf("TEST FAILED\n");
                exit(1);
            }
        }
    }


    /* Delete the data arrays. */
    fftw_free( x_re );
    fftw_free( x_im );
    fftw_free( y_re );
    fftw_free( y_im );
    printf("TEST PASSED\n");
    return 0;
}

static double areldiff(double ar,double ai,double br,double bi)
{
    /* Use L1 norm for simplicity. */
    double adiff = fabs(ar-br) + fabs(ai-bi);
    double anorm = fabs(ar) + fabs(ai) + fabs(br) + fabs(bi);
    return adiff ? 2*adiff/anorm : adiff;
}
