/*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using FFTW3 interface to Intel MKL (C-interface).
!
!       Double precision split complex in-place 3D transform
!       using fftw_plan_guru_split_dft().
!
!****************************************************************************/

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "fftw3.h"

/* Helper: a relative difference between two complex numbers. */
static double areldiff(double ar,double ai,double br,double bi);

int main(void)
{
    /* Sizes of 3D transform. */
    int N1 = 3, N2 = 4, N3 = 5;

    fftw_plan plan;
    double *x_re, *x_im;
    int n1,n2,n3;

    printf("Double precision split complex in-place\n"
           " 3D transform %dx%dx%d\n"
           " using fftw_plan_guru_split_dft()\n",
           N1,N2,N3);

    /* Allocate i/o arrays. */
    {
        size_t sz = sizeof(double) * N1 * N2 * N3;
        x_re = (double*) fftw_malloc( sz );
        x_im = (double*) fftw_malloc( sz );
        assert(x_re);
        assert(x_im);
    }

    /* Create forward transform plan. */
    {
        fftw_iodim dim[3];

        /* Note: we set up row-major layout:
         * X(n1,n2,n3) = X[n1*N2*N3+n2*N3+n3].
         */
        dim[0].n = N1; dim[0].is = dim[0].os = 1 * N3 * N2;
        dim[1].n = N2; dim[1].is = dim[1].os = 1 * N3;
        dim[2].n = N3; dim[2].is = dim[2].os = 1;

        plan = fftw_plan_guru_split_dft(3, dim, 0, NULL,
                                        x_re, x_im, x_re, x_im,
                                        FFTW_ESTIMATE);
        assert(plan);
    }

    /* Enter input data. */
    /* Note: this should be done after the plan is created. */
    for (n1 = 0; n1 < N1; ++n1)
    {
        for (n2 = 0; n2 < N2; ++n2)
        {
            for (n3 = 0; n3 < N3; ++n3)
            {
                x_re[n1*N2*N3 + n2*N3 + n3] = sin(n1+n2+n3+0.1);
                x_im[n1*N2*N3 + n2*N3 + n3] = sin(n1+n2+n3-0.1);
            }
        }
    }

    /* Compute forward transform. */
    fftw_execute(plan);

    /* Compute backward transform with new-array function. */
    /* Note: we use the same plan, but exchange Re<->Im. */
    fftw_execute_split_dft(plan,x_im,x_re,x_im,x_re);

    /* Destroy the plan. */
    fftw_destroy_plan(plan);

    /* Check the result: should be simply scaled input. */
    for (n1 = 0; n1 < N1; ++n1)
    {
        for (n2 = 0; n2 < N2; ++n2)
        {
            for (n3 = 0; n3 < N3; ++n3)
            {
                double got_re = x_re[n1*N2*N3 + n2*N3 + n3];
                double got_im = x_im[n1*N2*N3 + n2*N3 + n3];
                double exd_re = sin(n1+n2+n3+0.1) * N1 * N2 * N3;
                double exd_im = sin(n1+n2+n3-0.1) * N1 * N2 * N3;
                if (!(areldiff(got_re,got_im,exd_re,exd_im) < 1e-8))
                {
                    printf("Result(%d,%d,%d): "
                           "expected (%lg,%lg), got (%lg,%lg)\n",
                           n1, n2, n3, exd_re, exd_im, got_re, got_im);
                    printf("TEST FAILED\n");
                    exit(1);
                }
            }
        }
    }

    /* Delete the data arrays. */
    fftw_free( x_re );
    fftw_free( x_im );
    printf("TEST PASSED\n");
    return 0;
}

static double areldiff(double ar,double ai,double br,double bi)
{
    /* Use L1 norm for simplicity. */
    double adiff = fabs(ar-br) + fabs(ai-bi);
    double anorm = fabs(ar) + fabs(ai) + fabs(br) + fabs(bi);
    return adiff ? 2*adiff/anorm : adiff;
}
