/*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using FFTW3 interface to Intel MKL (C-interface).
!
!       Single precision multiple split complex out-of-place 3D transform
!       with non-unit strides using fftwf_plan_guru64_split_dft().
!
!****************************************************************************/

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "fftw3.h"

#define MAX2(a,b)     ((a)>(b)?(a):(b))
#define MAX4(a,b,c,d) MAX2(MAX2(a,b),MAX2(c,d))

/* Helper: a relative difference between two complex numbers. */
static float areldiff(float ar,float ai,float br,float bi);

int main(void)
{
    int N1 = 3, N2 = 4, N3 = 5; /* Sizes of 3D transform. */
    int M = 2;                  /* Number of transforms. */

    /* Stride and distance for x:
     * x_re(n1,n2,n3,m) = x_re[n1*XS1 + n2*XS2 + n3*XS3 + m*XD]
     */
    int XS1 = 1, XS2 = 1*N1, XS3 = 1*N1*N2, XD = 1*N1*N2*N3;

    /* Stride and distance for y:
     * y_re(n1,n2,n3,m) = y_re[n1*YS1 + n2*YS2 + n3*YS3 + m*YD]
     */
    int YS1 = N2*N3*M*2, YS2 = N3*M*2, YS3 = M*2, YD = 2;

    fftwf_plan planf,planb;
    float *x_re, *x_im, *y_re, *y_im;
    int n1,n2,n3,m;

    printf("Single precision multiple split complex out-of-place\n"
           " 3D transform %d:%d:%dx%d:%d:%dx%d:%d:%d*%d:%d:%d\n"
           " using fftwf_plan_guru64_split_dft()\n",
           N1,XS1,YS1,N2,XS2,YS2,N3,XS3,YS3,M,XD,YD);

    /* Allocate i/o arrays. */
    {
        size_t xsz = sizeof(float) * MAX4(N1*XS1, N2*XS2, N3*XS3, M*XD);
        size_t ysz = sizeof(float) * MAX4(N1*YS1, N2*YS2, N3*YS3, M*YD);
        x_re = (float*) fftwf_malloc( xsz );
        x_im = (float*) fftwf_malloc( xsz );
        y_re = (float*) fftwf_malloc( ysz );
        y_im = (float*) fftwf_malloc( ysz );
        assert(x_re);
        assert(x_im);
        assert(y_re);
        assert(y_im);
    }

    /* Create forward transform plan. */
    {
        fftwf_iodim64 dim[3], howmany;

        dim[0].n = N1; dim[0].is = XS1; dim[0].os = YS1;
        dim[1].n = N2; dim[1].is = XS2; dim[1].os = YS2;
        dim[2].n = N3; dim[2].is = XS3; dim[2].os = YS3;

        howmany.n = M;
        howmany.is = XD;
        howmany.os = YD;

        planf = fftwf_plan_guru64_split_dft(3, dim, 1, &howmany,
                                           x_re, x_im, y_re, y_im,
                                           FFTW_ESTIMATE);
        assert(planf);
    }

    /* Create backward transform plan. */
    {
        fftwf_iodim64 dim[3], howmany;

        dim[0].n = N1; dim[0].is = YS1; dim[0].os = XS1;
        dim[1].n = N2; dim[1].is = YS2; dim[1].os = XS2;
        dim[2].n = N3; dim[2].is = YS3; dim[2].os = XS3;

        howmany.n = M;
        howmany.is = YD;
        howmany.os = XD;

        planb = fftwf_plan_guru64_split_dft(3, dim, 1, &howmany,
                                           y_im, y_re, x_im, x_re,
                                           FFTW_ESTIMATE);
        assert(planb);
    }

    /* Enter input data. */
    /* Note: input should be initialized after the plan is created. */
    for (m = 0; m < M; ++m)
    {
        for (n1 = 0; n1 < N1; ++n1)
        {
            for (n2 = 0; n2 < N2; ++n2)
            {
                for (n3 = 0; n3 < N3; ++n3)
                {
                    x_re[n1*XS1 + n2*XS2 + n3*XS3 + m*XD] = sinf(n1+n2+n3+0.1f);
                    x_im[n1*XS1 + n2*XS2 + n3*XS3 + m*XD] = sinf(n1+n2+n3-0.1f);
                }
            }
        }
    }

    /* Compute forward transform. */
    fftwf_execute(planf);

    /* Compute backward transform. */
    fftwf_execute(planb);

    /* Destroy the plans. */
    fftwf_destroy_plan(planf);
    fftwf_destroy_plan(planb);

    /* Check the result: should be simply scaled input. */
    for (m = 0; m < M; ++m)
    {
        for (n1 = 0; n1 < N1; ++n1)
        {
            for (n2 = 0; n2 < N2; ++n2)
            {
                for (n3 = 0; n3 < N3; ++n3)
                {
                    float got_re = x_re[n1*XS1 + n2*XS2 + n3*XS3 + m*XD];
                    float got_im = x_im[n1*XS1 + n2*XS2 + n3*XS3 + m*XD];
                    float exd_re = sinf(n1+n2+n3+0.1f) * N1 * N2 * N3;
                    float exd_im = sinf(n1+n2+n3-0.1f) * N1 * N2 * N3;
                    if (!(areldiff(got_re,got_im,exd_re,exd_im) < 1e-4f))
                    {
                        printf("Result(%d,%d,%d;%d): "
                               "expected (%lg,%lg), got (%lg,%lg)\n",
                               n1, n2, n3, m, exd_re, exd_im, got_re, got_im);
                        printf("TEST FAILED\n");
                        exit(1);
                    }
                }
            }
        }
    }

    /* Delete the data arrays. */
    fftwf_free( x_re );
    fftwf_free( x_im );
    fftwf_free( y_re );
    fftwf_free( y_im );
    printf("TEST PASSED\n");
    return 0;
}

static float areldiff(float ar,float ai,float br,float bi)
{
    /* Use L1 norm for simplicity. */
    float adiff = fabsf(ar-br) + fabsf(ai-bi);
    float anorm = fabsf(ar) + fabsf(ai) + fabsf(br) + fabsf(bi);
    return adiff ? 2*adiff/anorm : adiff;
}
