!*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using FFTW3 Fortran interface to Intel MKL.
!
!       Double precision split complex in-place 3D transform
!       using dfftw_plan_guru_split_dft().
!
!*****************************************************************************

program double_split_complex_3d_ex1

  include 'fftw3.f'
  integer, parameter :: N(3) = (/16,32,8/) ! Sizes of 3D transform

  integer*8 plan
  real*8, allocatable :: xr(:,:,:), xi(:,:,:)
  complex*16 :: got, exd
  integer n1, n2, n3, stride(3)

  print *,"Double precision split complex in-place"
  print '(" 3D transform "I0"x"I0"x"I0)', N
  print *,"using dfftw_plan_guru_split_dft()"

  ! Allocate i/o arrays.
  allocate(xr(N(1),N(2),N(3)),xi(N(1),N(2),N(3)))

  ! Create forward transform plan.
  stride(1) = 1
  stride(2) = size(xr,dim=1)
  stride(3) = size(xr,dim=1)*size(xr,dim=2)

  call dfftw_plan_guru_split_dft(plan, &
       & 3, N, stride, stride, &
       & 0, 0, 0, 0, &
       & xr, xi, xr, xi, FFTW_ESTIMATE)
  if (plan == 0) stop 1

  ! Enter input data.
  ! Note: this should be done after plan is created.
  do n1 = 1, N(1)
    do n2 = 1, N(2)
      do n3 = 1, N(3)
        xr(n1,n2,n3) = sin(n1+n2+n3+0.1d0)
        xi(n1,n2,n3) = sin(n1+n2+n3-0.1d0)
      enddo
    enddo
  enddo

  ! Compute forward transform.
  call dfftw_execute(plan)

  ! Compute backward transform with new-array function.
  ! Note: we use the same plan, but exchange Re<->Im.
  call dfftw_execute_split_dft(plan,xi,xr,xi,xr)

  ! Destroy the plan.
  call dfftw_destroy_plan(plan)

  ! Check the result: should be simply scaled input.
  do n1 = 1, N(1)
     do n2 = 1, N(2)
        do n3 = 1, N(3)
           got = dcmplx(xr(n1,n2,n3),xi(n1,n2,n3))
           exd = dcmplx(sin(n1+n2+n3+0.1d0),sin(n1+n2+n3-0.1d0)) &
               & *N(1)*N(2)*N(3)
           if (.not.(areldiff(got,exd) < 1d-8)) then
              call error(n1,n2,n3,exd,got)
           endif
        enddo
     enddo
  enddo

  ! Deallocate data arrays.
  deallocate( xr, xi )
  print *, 'TEST PASSED'
  stop 0

contains

  ! A relative difference between two complex numbers
  real*8 function areldiff(a,b)
    complex*16 a,b
    real*8 norm
    norm = max(abs(a),abs(b))
    if (norm > 0) then
       areldiff = abs(a-b) / norm
    else
       areldiff = abs(a-b)
    endif
  end function areldiff

  ! Report error and terminate
  subroutine error(n1,n2,n3,exd,got)
    integer :: n1,n2,n3
    complex*16 :: exd, got

    print '("Result("I0","I0","I0"): "$)', n1,n2,n3
    print '("expected ("G15.8","G15.8") "$)', exd
    print '("got ("G15.8","G15.8")")', got
    print *, 'TEST FAILED'
    stop 1
  end subroutine error

end program double_split_complex_3d_ex1
