!*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using FFTW3 Fortran interface to Intel MKL.
!
!       Double precision multiple split complex out-of-place 3D transform
!       using dfftw_plan_guru_split_dft().
!
!*****************************************************************************

program double_split_complex_3d_ex2

  include 'fftw3.f'
  integer, parameter :: N(3) = (/5,10,20/) ! Sizes of 3D transform
  integer, parameter :: M = 4 ! Number of transforms

  integer*8 planf, planb
  real*8, allocatable :: xr(:,:,:,:), xi(:,:,:,:)
  real*8, allocatable :: yr(:,:,:,:), yi(:,:,:,:)
  complex*16 :: got, exd
  integer n1, n2, n3, mm, xstride(3), ystride(3), xdist, ydist

  ! Allocate i/o arrays.
  allocate(xr(M,N(1),N(2),N(3)),xi(M,N(1),N(2),N(3)))
  allocate(yr(N(1),N(2),N(3),M),yi(N(1),N(2),N(3),M))

  ! Set up stride arrays needed for plan creation
  xdist      = 1
  xstride(1) = size(xr,dim=1)
  xstride(2) = size(xr,dim=1)*size(xr,dim=2)
  xstride(3) = size(xr,dim=1)*size(xr,dim=2)*size(xr,dim=3)
  ystride(1) = 1
  ystride(2) = size(yr,dim=1)
  ystride(3) = size(yr,dim=1)*size(yr,dim=2)
  ydist      = size(yr,dim=1)*size(yr,dim=2)*size(yr,dim=3)

  print *,"Double precision split complex in-place"
  print '(" 3D transform "I0":"I0":"I0"x"I0":"I0":"I0"x"I0":"I0":"I0$)', &
       & N(1), xstride(1), ystride(1), &
       & N(2), xstride(2), ystride(2), &
       & N(3), xstride(3), ystride(3)
  print '("*"I0":"I0":"I0)', M, xdist, ydist
  print *,"using dfftw_plan_guru_split_dft()"

  ! Create forward transform plan.
  call dfftw_plan_guru_split_dft(planf, &
       & 3, N, xstride, ystride, &
       & 1, M, xdist,   ydist, &
       & xr, xi, yr, yi, FFTW_ESTIMATE)
  if (planf == 0) stop 1

  ! Create backward transform plan.
  call dfftw_plan_guru_split_dft(planb, &
       & 3, N, ystride, xstride, &
       & 1, M, ydist,   xdist, &
       & yi, yr, xi, xr, FFTW_ESTIMATE)
  if (planb == 0) stop 1

  ! Enter input data.
  ! Note: this should be done after plan is created.
  do n1 = 1, N(1)
    do n2 = 1, N(2)
      do n3 = 1, N(3)
        xr(:,n1,n2,n3) = sin(n1+n2+n3+0.1d0)
        xi(:,n1,n2,n3) = sin(n1+n2+n3-0.1d0)
      enddo
    enddo
  enddo

  ! Compute forward transform.
  call dfftw_execute(planf)

  ! Compute backward transform.
  call dfftw_execute(planb)

  ! Destroy the plans.
  call dfftw_destroy_plan(planf)
  call dfftw_destroy_plan(planb)

  ! Check the result: should be simply scaled input.
  do n1 = 1, N(1)
     do n2 = 1, N(2)
        do n3 = 1, N(3)
           do mm = 1, M
              got = dcmplx(xr(mm,n1,n2,n3),xi(mm,n1,n2,n3))
              exd = dcmplx(sin(n1+n2+n3+0.1d0),sin(n1+n2+n3-0.1d0)) &
                  & *N(1)*N(2)*N(3)
              if (.not.(areldiff(got,exd) < 1d-8)) then
                 call error(mm,n1,n2,n3,exd,got)
              endif
           enddo
        enddo
     enddo
  enddo

  ! Deallocate data arrays.
  deallocate( xr, xi )
  deallocate( yr, yi )
  print *, 'TEST PASSED'
  stop 0

contains

  ! A relative difference between two complex numbers
  real*8 function areldiff(a,b)
    complex*16 a,b
    real*8 norm
    norm = max(abs(a),abs(b))
    if (norm > 0) then
       areldiff = abs(a-b) / norm
    else
       areldiff = abs(a-b)
    endif
  end function areldiff

  ! Report error and terminate
  subroutine error(mm,n1,n2,n3,exd,got)
    integer :: mm,n1,n2,n3
    complex*16 :: exd, got

    print '("Result("I3";"I3","I3","I3"): "$)', mm,n1,n2,n3
    print '("expected ("G15.8","G15.8") "$)', exd
    print '("got ("G15.8","G15.8")")', got
    print *, 'TEST FAILED'
    stop 1
  end subroutine error

end program double_split_complex_3d_ex2
