!*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using FFTW3 Fortran interface to Intel MKL.
!
!       Single precision multiple split complex out-of-place 1D transform
!       using sfftw_plan_guru_split_dft().
!
!*****************************************************************************

program single_split_complex_1d_ex2

  include 'fftw3.f'
  integer, parameter :: N = 10 ! Size of transform
  integer, parameter :: M = 8  ! Number of transforms

  integer*8 planf, planb
  real*4, allocatable :: xr(:,:), xi(:,:), yr(:,:), yi(:,:)
  complex*8 :: got, exd
  integer i, j

  print *,"Single precision multiple split complex out-of-place"
  print '(" 1D transform "I0":"I0":"I0"*"I0":"I0":"I0)', N,1,M, M,N,1
  print *,"using sfftw_plan_guru_split_dft()"

  ! Allocate i/o arrays.
  allocate(xr(N,M),xi(N,M))
  allocate(yr(M,N),yi(M,N))

  ! Create plan for forward transform: X-->Y
  call sfftw_plan_guru_split_dft(planf, &
       & 1, N, 1, M, &
       & 1, M, N, 1, &
       & xr, xi, yr, yi, FFTW_ESTIMATE)
  if (planf == 0) stop 1

  ! Create plan for backward transform: Y-->X
  call sfftw_plan_guru_split_dft(planb, &
       & 1, N, M, 1, &
       & 1, M, 1, N, &
       & yi, yr, xi, xr, FFTW_ESTIMATE)
  if (planb == 0) stop 1

  ! Enter input data.
  ! Note: this should be done after plan is created.
  do i = 1, N
    xr(i,:) = sin(i+0.1)
    xi(i,:) = sin(i-0.1)
  enddo

  ! Compute forward transform.
  call sfftw_execute(planf)

  ! Compute backward transform with new-array function.
  call sfftw_execute(planb)

  ! Destroy the plans.
  call sfftw_destroy_plan(planf)
  call sfftw_destroy_plan(planb)

  ! Check the result: should be simply scaled input.
  do i = 1, N
     do j = 1, M
        got = cmplx(xr(i,j),xi(i,j))
        exd = cmplx(sin(i+0.1),sin(i-0.1))*N
        if (.not.(areldiff(got,exd) < 1e-4)) then
           call error(i,j,exd,got)
        endif
     enddo
  enddo


  ! Deallocate data arrays.
  deallocate( xr, xi )
  deallocate( yr, yi )
  print *, 'TEST PASSED'
  stop 0

contains

  ! A relative difference between two complex numbers
  real*4 function areldiff(a,b)
    complex*8 a,b
    real*4 norm
    norm = max(abs(a),abs(b))
    if (norm > 0) then
       areldiff = abs(a-b) / norm
    else
       areldiff = abs(a-b)
    endif
  end function areldiff

  ! Report error and terminate
  subroutine error(i,j,exd,got)
    integer :: i,j
    complex*8 :: exd, got

    print '("Result("I0";"I0"): "$)', i,j
    print '("expected ("G11.4","G11.4") "$)', exd
    print '("got ("G11.4","G11.4")")', got
    print *, 'TEST FAILED'
    stop 1
  end subroutine error

end program single_split_complex_1d_ex2
