!*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using FFTW3 Fortran interface to Intel MKL.
!
!       Single precision split complex in-place 2D transform
!       using sfftw_plan_guru_split_dft().
!
!*****************************************************************************

program single_split_complex_2d_ex1

  include 'fftw3.f'
  integer, parameter :: N(2) = (/10,20/) ! Sizes of 2D transform

  integer*8 plan
  real*4, allocatable :: xr(:,:), xi(:,:)
  complex*8 :: got, exd
  integer n1, n2, stride(2)

  print *,"Single precision split complex in-place"
  print '(" 2D transform "I0"x"I0)', N
  print *,"using sfftw_plan_guru_split_dft()"

  ! Allocate i/o arrays.
  allocate(xr(N(1),N(2)),xi(N(1),N(2)))

  ! Create forward transform plan.
  stride(1) = 1
  stride(2) = size(xr,dim=1)

  call sfftw_plan_guru_split_dft(plan, &
       & 2, N, stride, stride, &
       & 0, 0, 0, 0, &
       & xr, xi, xr, xi, FFTW_ESTIMATE)
  if (plan == 0) stop 1

  ! Enter input data.
  ! Note: this should be done after plan is created.
  do n1 = 1, N(1)
    do n2 = 1, N(2)
      xr(n1,n2) = sin(n1+n2+0.1)
      xi(n1,n2) = sin(n1+n2-0.1)
    enddo
  enddo

  ! Compute forward transform.
  call sfftw_execute(plan)

  ! Compute backward transform with new-array function.
  ! Note: we use the same plan, but exchange Re<->Im.
  call sfftw_execute_split_dft(plan,xi,xr,xi,xr)

  ! Destroy the plan.
  call sfftw_destroy_plan(plan)

  ! Check the result: should be simply scaled input.
  do n1 = 1, N(1)
     do n2 = 1, N(2)
        got = cmplx(xr(n1,n2),xi(n1,n2))
        exd = cmplx(sin(n1+n2+0.1),sin(n1+n2-0.1))*N(1)*N(2)
        if (.not.(areldiff(got,exd) < 1e-4)) then
           call error(n1,n2,exd,got)
        endif
     enddo
  enddo


  ! Deallocate data arrays.
  deallocate( xr, xi )
  print *, 'TEST PASSED'
  stop 0

contains

  ! A relative difference between two complex numbers
  real*4 function areldiff(a,b)
    complex*8 a,b
    real*4 norm
    norm = max(abs(a),abs(b))
    if (norm > 0) then
       areldiff = abs(a-b) / norm
    else
       areldiff = abs(a-b)
    endif
  end function areldiff

  ! Report error and terminate
  subroutine error(n1,n2,exd,got)
    integer :: n1,n2
    complex*8 :: exd, got
    
    print '("Result("I0","I0"): "$)', n1,n2
    print '("expected ("G11.4","G11.4") "$)', exd
    print '("got ("G11.4","G11.4")")', got
    print *, 'TEST FAILED'
    stop 1
  end subroutine error

end program single_split_complex_2d_ex1
