!*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using FFTW3 Fortran interface to Intel MKL.
!
!       Single precision multiple split complex out-of-place 2D transform
!       with non-unit strides using sfftw_plan_guru_split_dft().
!
!*****************************************************************************

program single_split_complex_2d_ex2

  include 'fftw3.f'
  integer, parameter :: N(2) = (/20,30/) ! Sizes of 2D transform
  integer, parameter :: M = 4   ! Number of transforms

  integer*8 planf, planb
  real*4, allocatable :: xr(:,:,:), xi(:,:,:)
  real*4, allocatable :: yr(:,:,:), yi(:,:,:)
  complex*8 :: got, exd
  integer n1, n2, mm, xstride(2), ystride(2), xdist, ydist

  ! Allocate i/o arrays.
  allocate(xr(N(1),N(2),M),xi(N(1),N(2),M))
  allocate(yr(M,N(2),N(1)),yi(M,N(2),N(1)))

  ! Set up stride arrays needed for plan creation
  xstride(1) = 1
  xstride(2) = size(xr,dim=1)
  xdist      = size(xr,dim=1)*size(xr,dim=2)
  ydist      = 1
  ystride(2) = size(yr,dim=1)
  ystride(1) = size(yr,dim=1)*size(yr,dim=2)

  print *,"Single precision multiple split complex out-of-place"
  print '(" 2D transform "I0":"I0":"I0"x"I0":"I0":"I0"*"I0":"I0":"I0)', &
       & N(1),xstride(1),ystride(1),N(2),xstride(2),ystride(2), &
       & M,xdist,ydist
  print *,"using sfftw_plan_guru_split_dft()"

  ! Create plan for forward transform: X-->Y
  call sfftw_plan_guru_split_dft(planf, &
       & 2, N, xstride, ystride, &
       & 1, M, xdist,   ydist, &
       & xr, xi, yr, yi, FFTW_ESTIMATE)
  if (planf == 0) stop 1

  ! Create plan for backward transform: Y->X
  call sfftw_plan_guru_split_dft(planb, &
       & 2, N, ystride, xstride, &
       & 1, M, ydist,   xdist, &
       & yi, yr, xi, xr, FFTW_ESTIMATE)
  if (planb == 0) stop 1

  ! Enter input data.
  ! Note: this should be done after plan is created.
  do n1 = 1, N(1)
    do n2 = 1, N(2)
      xr(n1,n2,:) = sin(n1+n2+0.1)
      xi(n1,n2,:) = sin(n1+n2-0.1)
    enddo
  enddo

  ! Compute forward transform.
  call sfftw_execute(planf)

  ! Compute backward transform.
  call sfftw_execute(planb)

  ! Destroy the plans.
  call sfftw_destroy_plan(planf)
  call sfftw_destroy_plan(planb)

  ! Check the result: should be simply scaled input.
  do n1 = 1, N(1)
     do n2 = 1, N(2)
        do mm = 1, M
           got = cmplx(xr(n1,n2,mm),xi(n1,n2,mm))
           exd = cmplx(sin(n1+n2+0.1), sin(n1+n2-0.1))*N(1)*N(2)
           if (.not.(areldiff(got,exd) < 1e-4)) then
              call error(n1,n2,mm,exd,got)
           endif
        enddo
     enddo
  enddo

  ! Deallocate data arrays.
  deallocate( xr, xi )
  deallocate( yr, yi )
  print *, 'TEST PASSED'
  stop 0

contains

  ! A relative difference between two complex numbers
  real*4 function areldiff(a,b)
    complex*8 a,b
    real*4 norm
    norm = max(abs(a),abs(b))
    if (norm > 0) then
       areldiff = abs(a-b) / norm
    else
       areldiff = abs(a-b)
    endif
  end function areldiff

  ! Report error and terminate
  subroutine error(n1,n2,mm,exd,got)
    integer :: n1,n2,mm
    complex*8 :: exd, got

    print '("Result("I3","I3";"I3"): "$)', n1,n2,mm
    print '("expected ("G11.4","G11.4") "$)', exd
    print '("got ("G11.4","G11.4")")', got
    print *, 'TEST FAILED'
    stop 1
  end subroutine error

end program single_split_complex_2d_ex2
