/******************************************************************************
 *                            INTEL CONFIDENTIAL
 * Copyright(C) 2006-2010 Intel Corporation. All Rights Reserved.
 * The source code contained  or  described herein and all documents related to
 * the source code ("Material") are owned by Intel Corporation or its suppliers
 * or licensors.  Title to the  Material remains with  Intel Corporation or its
 * suppliers and licensors. The Material contains trade secrets and proprietary
 * and  confidential  information of  Intel or its suppliers and licensors. The
 * Material  is  protected  by  worldwide  copyright  and trade secret laws and
 * treaty  provisions. No part of the Material may be used, copied, reproduced,
 * modified, published, uploaded, posted, transmitted, distributed or disclosed
 * in any way without Intel's prior express written permission.
 * No license  under any  patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or delivery
 * of the Materials,  either expressly, by implication, inducement, estoppel or
 * otherwise.  Any  license  under  such  intellectual property  rights must be
 * express and approved by Intel in writing.
 *
 ******************************************************************************
 *
 * fftwnd - FFTW2 wrapper to MKL.
 *
 ******************************************************************************
 */

#include "fftw2_mkl.h"

void
fftwnd(fftwnd_plan _plan, int howmany, fftw_complex *in, int istride, int idist,
       fftw_complex *out, int ostride, int odist)
{
    MKL_LONG err; /* in DFTI operations */
    int need_commit = 0;
    fftw_plan_mkl *plan = (fftw_plan_mkl *) _plan;


    if (plan->howmany != howmany)
    {
        need_commit = 1;
        if (plan->readonly) goto cannot_commit;
        plan->howmany = howmany;
        err = DftiSetValue(plan->mkl_desc, DFTI_NUMBER_OF_TRANSFORMS,
                plan->howmany);
        if (err) goto cannot_commit;
    }

    if (plan->istride != istride)
    {
        MKL_LONG strides[1 + DFTI_MAX_RANK];
        int rank = plan->rank, i;

        if (plan->readonly) goto cannot_commit;
        strides[0] = 0;
        strides[rank] = plan->istride = istride;
        for (i = rank - 1; i > 0; i--)
        {
            strides[i] = plan->n[i] * strides[i + 1];
        }
        err = DftiSetValue(plan->mkl_desc, DFTI_INPUT_STRIDES, strides);
        if (err) goto cannot_commit;
    }
        
    if ((plan->idist != idist) && (howmany > 1))
    {
        need_commit = 1;
        if (plan->readonly) goto cannot_commit;
        plan->idist = idist;
        err = DftiSetValue(plan->mkl_desc, DFTI_INPUT_DISTANCE, plan->idist);
        if (err) goto cannot_commit;
    }

    /* According to FFTW2 documentation, if plan is inplace,
     * then parameters out, ostride, and odist are ignored.
     */
    if (!plan->inplace)
    {
        if (plan->ostride != ostride)
        {
            MKL_LONG strides[1 + DFTI_MAX_RANK];
            int rank = plan->rank, i;

            need_commit = 1;
            if (plan->readonly) goto cannot_commit;
            strides[0] = 0;
            strides[rank] = plan->ostride = ostride;
            for (i = rank - 1; i > 0; i--)
            {
                strides[i] = plan->n[i] * strides[i + 1];
            }
            err = DftiSetValue(plan->mkl_desc, DFTI_OUTPUT_STRIDES, strides);
            if (err) goto cannot_commit;
        }

        if ((plan->odist != odist) && (howmany > 1))
        {
            need_commit = 1;
            if (plan->readonly) goto cannot_commit;
            plan->odist = odist;
            err = DftiSetValue(plan->mkl_desc, DFTI_OUTPUT_DISTANCE, plan->odist);
            if (err) goto cannot_commit;
        }
    }

    if (need_commit)
    {
        if (plan->readonly) goto cannot_commit;
        err = DftiCommitDescriptor(plan->mkl_desc);
        if (err) goto cannot_commit;
    }

    if (plan->sign == FFTW_FORWARD)
    {
        err = DftiComputeForward(plan->mkl_desc, in, out);
    }
    else
    {
        err = DftiComputeBackward(plan->mkl_desc, in, out);
    }
    if (err) fftw_die("DftiCompute returned error in fftwnd()");
    return;

cannot_commit:
    fftw_die("Cannot commit plan in fftwnd()");
}
